@@ -1,6 +1,6 @@
 /********************************************
 init.h
-copyright 2009-2016,2020, Thomas E. Dickey
+copyright 2009-2024,2026, Thomas E. Dickey
 copyright 1991, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: init.h,v 1.7 2020/08/26 00:52:06 tom Exp $
+ * $MawkId: init.h,v 1.10 2026/01/16 23:53:35 tom Exp $
  */
 
 /* init.h  */
@@ -19,15 +19,20 @@ the GNU General Public License, version 2, 1991.
 #ifndef  INIT_H
 #define  INIT_H
 
-#include "symtype.h"
+#include <stdio.h>
+#include <symtype.h>
 
 /* nodes to link file names for multiple
    -f option */
 
-typedef struct pfile {
-    struct pfile *link;
+typedef struct _pfile
+#ifdef Visible_PFILE
+{
+    struct _pfile *link;
     char *fname;
-} PFILE;
+}
+#endif
+PFILE;
 
 extern PFILE *pfile_list;
 
@@ -47,7 +52,7 @@ void fpe_init(void);
 void load_environ(ARRAY);
 void set_stdio(void);
 
-void print_version(FILE *fp);
+ GCC_NORETURN void print_version(FILE *fp);
 int is_cmdline_assign(char *);
 
 #endif /* INIT_H  */
