@@ -1,6 +1,6 @@
 /********************************************
 split.c
-copyright 2008-2014,2020 Thomas E. Dickey
+copyright 2008-2020,2024 Thomas E. Dickey
 copyright 1991-1996,2014 Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,27 +11,31 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: split.c,v 1.32 2020/09/13 14:41:05 tom Exp $
+ * $MawkId: split.c,v 1.37 2024/12/14 12:53:38 tom Exp $
  */
 
-/* split.c */
+#define Visible_BI_REC
+#define Visible_CELL
+#define Visible_RE_DATA
+#define Visible_STRING
 
-#include "mawk.h"
-#include "split.h"
-#include "symtype.h"
-#include "bi_vars.h"
-#include "bi_funct.h"
-#include "memory.h"
-#include "scan.h"
-#include "regexp.h"
-#include "repl.h"
-#include "field.h"
+#include <mawk.h>
+#include <split.h>
+#include <symtype.h>
+#include <bi_vars.h>
+#include <bi_funct.h>
+#include <memory.h>
+#include <scan.h>
+#include <regexp.h>
+#include <field.h>
 
+#ifndef SP_SIZE
 #ifdef NO_LEAKS
 #define SP_SIZE    4		/* exercises split_block_list code */
 #else
 #define SP_SIZE  2048
 #endif
+#endif
 
 typedef struct split_block {
     STRING *strings[SP_SIZE];
@@ -50,7 +54,7 @@ static void
 spb_list_shrink(void)
 {
     Split_Block_Node *p = split_block_list->link;
-    split_block_list->link = 0;
+    split_block_list->link = NULL;
     while (p) {
 	Split_Block_Node *hold = p;
 	p = p->link;
@@ -68,13 +72,13 @@ grow_sp_list(Split_Block_Node * tail)
 {
     tail->link = (Split_Block_Node *) zmalloc(sizeof(Split_Block_Node));
     tail = tail->link;
-    tail->link = 0;
+    tail->link = NULL;
     return tail;
 }
 
 /*
  * Split string s of length slen on SPACE without changing s.
- * Load the pieces into STRINGS 
+ * Load the pieces into STRINGS
  * return the number of pieces
  */
 size_t
@@ -156,7 +160,7 @@ re_split(char *s, size_t slen, PTR re)
  * length of match is returned in *lenp
  */
 char *
-re_pos_match(char *str, size_t str_len, PTR re, size_t *lenp, int no_bol)
+re_pos_match(char *str, size_t str_len, RE_NODE * re, size_t *lenp, int no_bol)
 {
     const char *end = str + str_len;
 
@@ -176,11 +180,11 @@ re_pos_match(char *str, size_t str_len, PTR re, size_t *lenp, int no_bol)
 	} else {
 	    /* no match */
 	    *lenp = 0;
-	    return 0;
+	    return NULL;
 	}
     }
     *lenp = 0;
-    return 0;
+    return NULL;
 }
 
 /*
